/*
 * Symbol.java
 *
 * Created on February 18, 2002, 9:32 AM
 */

package org.semanticweb.dl;

import java.io.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.jena.daml.common.VocabularyManager;

/**
 * A (DL) symbol (for classes and roles), wrapping a resource.
 * @author  Michael Sintek
 * @version 
 */
public class Symbol extends Expression {

  Resource _resource; // not used at the moment
  
  String _namespace;
  String _name;
  
  boolean _isTop = false;
  boolean _isBottom = false;
  
  public static final String DAML_NAMESPACE =
    new VocabularyManager().getDefaultVocabulary().NAMESPACE_DAML().getNameSpace();
  static final int _bottom = 0;
  static final int _top = 1;
  static final int _unknown = 2;
  
  public static Symbol BOTTOM = new Symbol(_bottom);
  public static Symbol TOP = new Symbol(_top);
  public static Symbol UNKNOWN = new Symbol(_unknown);

  
  /** Creates new Symbol */
  public Symbol(Resource resource) {
    _resource = resource;
    _namespace = resource.getNameSpace();
    if (_namespace == null) _namespace = "";
    _name = resource.getLocalName();
    if (_name == null) _name = "?"; // should never happen
    // ???:
    if (this.equals(TOP))         _isTop = true;
    // we also have to check for rdfs:Resource which is also == TOP!!!
    else if (this.equals(BOTTOM)) _isBottom = true;
  }
  
  public Symbol(String namespace, String name) {
    _namespace = namespace;
    _name = name;
    // check TOP/BOTTOM?
  }
  
  Symbol(int special) {
    _namespace = DAML_NAMESPACE;
    if (special == _top) {
      _name = "Thing";
      _isTop= true;
    } else if (special == _bottom) {
      _name = "Nothing";
      _isBottom = true;
    } else if (special == _unknown) {
      _name = "Unknown";
    }
  }

  public String getFullName() {
    return _namespace + _name;
  }
  
  public int compareTo(Object obj) {
    if (obj instanceof Symbol)
      return getFullName().compareTo(((Symbol)obj).getFullName());
    else
      return getFullName().compareTo(obj); // ???
  }
  
  public boolean isTop() {
    return _isTop;
  }
  
  public boolean isBottom() {
    return _isBottom;
  }
  
  public boolean equals(Object obj) {
    return ((obj instanceof Symbol) // no null values allowed!
      && _namespace.equals(((Symbol)obj)._namespace)
      && _name.equals(((Symbol)obj)._name));
  }

  public int hashCode() {
    return _name.hashCode(); // ???
  }
  
  void printSymbol(Syntax syntax, PrintWriter out) {
    if (syntax.TRIPLE()) {
      out.print("resource('" + _namespace + "','" + _name + "')");
    } else if (syntax.DAML()) { // no abbreviations
      out.print(_namespace + _name);
    } else {
      String abbrev = syntax._kb.getAbbreviation(_namespace);
      if (abbrev != null)
        out.print(abbrev + syntax.abbrevSplitter() + _name); // is + for LISP!
      else
        out.print(_namespace + _name); // normalize for XML/LISP dialects!!
    }
  }
  
  public void print(Syntax syntax, PrintWriter out) {
    if (_isTop) {
      if (syntax.XML())         out.print("<TOP/>");
      else if (syntax.TRIPLE()) out.print("resource('" + DAML_NAMESPACE + "','Thing')");
      else if (syntax.DOT())    printSymbol(syntax, out);
      else if (syntax.DAML())   printSymbol(syntax, out);
      else                      out.print("*top*");
    } else if (_isBottom) {
      if (syntax.XML())         out.print("<BOTTOM/>");
      else if (syntax.TRIPLE()) out.print("resource('" + DAML_NAMESPACE + "','Nothing')");
      else if (syntax.DOT())    printSymbol(syntax, out);
      else if (syntax.DAML())   printSymbol(syntax, out);
      else                      out.print("*bottom*");
    } else if (syntax.XML()) {
      String prim = null;
      if (syntax.isConcept(this))
        prim = "PRIMITIVE";
      else
        prim = "PRIMROLE";
      out.print("<" + prim + " NAME=\"");
      printSymbol(syntax, out);
      out.print("\"/>");
    // } else if (syntax.DOT()) {
    //   printSymbol(syntax, out);
    } else
      printSymbol(syntax, out);
  }
  
  public Expression toSHIQ(KB kb, int mode) {
    // this is the place where we really need the mode!
    if (mode == CONCEPT)
      kb.addConcept(this);
    else
      kb.addRole(this);
    return this;
  }

}
