/*
 * ExternalPrologCallback.java
 *
 * Created on February 9, 2002, 9:35 AM
 */

package org.semanticweb.triple.application;

import java.util.*;
import java.io.*;
import org.semanticweb.triple.parser.*;
import org.semanticweb.triple.language.Clause;
import org.semanticweb.glp.*;
import org.semanticweb.lp.*;
import org.w3c.rdf.*;
import org.w3c.rdf.util.*;
import org.w3c.rdf.model.*;
import org.w3c.rdf.implementation.model.*;
import org.w3c.rdf.syntax.*;
import org.xml.sax.InputSource;


/**
 * Implementation of Callback to generate output usable for
 * external Prologs like XSB.
 * @author  Michael Sintek
 * @version
 */
public class ExternalPrologCallback implements Callback, RDFConsumer {

  PrintWriter writer;
  Resource rdfNamespace = null;

  public ExternalPrologCallback() {
    this.writer = new PrintWriter(System.out);
  }

  public ExternalPrologCallback(PrintWriter writer) {
    this.writer = writer;
  }

  public void setRDFNamespace(Resource namespace){
    rdfNamespace = namespace;
  }


  public void nextClause(LTerm lclause, Context context) {
    LTerm lmodel = context.getModel();
    // writer.println("% HIL: " + lclause + " @ " + lmodel); // hilog representation
    try {
      // 1. native TRIPLE representation
      Clause clause = TripleTransform.transformClause(lclause, lmodel);
      writer.println("% TRP: " + clause);
      // if this is a query, also print in PROLOG
      // ... TODO ... !!!
      // 2. GLP
      GLPRule glpClause = (GLPRule)clause.toGLP(); // GLPClause ??
      writer.println("% GLP: " + glpClause);
      // 3. LP (Lloyd Topor application)
      GLPLloydToporTransform lt = new GLPLloydToporTransform();
      ArrayList lpRules = lt.translateFormula((GLPRule)glpClause);
      // 4. print as PROLOG program
      ArrayList answerVars = new ArrayList(); // of lp.VariableTerm
      ArrayList glpVars = glpClause.getVariables();
      if (glpVars != null) {
        for (Iterator varIter = glpVars.iterator(); varIter.hasNext();) {
          answerVars.add(((GLPTermVariable)varIter.next()).toLPTerm());
        }
      }
      for (Iterator i = lpRules.iterator(); i.hasNext();) {
        ProgramClause lpClause = (ProgramClause)i.next();
        writer.println(lpClause.toPrologString(answerVars));
      }
    } catch (Exception e) {
      writer.println("% ???: " + e);
      e.printStackTrace();
    }
    writer.println();
  }

  /**
   * start is called when parsing of data is started
   */
  public void startModel () throws ModelException {
  }
  
  /**
   * end is called when parsing of data is ended
   */
  public void endModel () throws ModelException{
  }
  
  /**
   * node factory to be used by the parser for creating resources and literals passed to addStatement
   */
  public NodeFactory getNodeFactory () throws ModelException{
    return new NodeFactoryImpl ();
  }
  
  /**
   * assert is called every time a new statement within
   * RDF data model is added
   */
  private FunctorTerm createResourceTerm (Resource r) throws ModelException {
    String namespace =  r.getNamespace ();
    String localName = r.getLocalName ();
    if(namespace == null) {
      namespace = RDFUtil.guessNamespace (r.getURI ());
      localName = RDFUtil.guessName (r.getURI ());
      //System.out.println("identified " + namespace + " " + localName);
    }
    Term[] resourceArguments = { new ConstTerm (namespace),  new ConstTerm (localName) };
    return new FunctorTerm ("resource",resourceArguments);
  }
  
  public void addStatement (Statement s) throws ModelException{
    Term[] tripleArgs = new Term[3];
    tripleArgs[0] = createResourceTerm (s.subject ());
    tripleArgs[1] = createResourceTerm (s.predicate ());
    if(s.object () instanceof Resource){
      tripleArgs[2] = createResourceTerm ((Resource) s.object ());
    } else
      tripleArgs[2] = new ConstTerm (s.object ().getLabel ());
    Term[] trueArgs = new Term[2];
    trueArgs[0] = new FunctorTerm ("triple", tripleArgs);
    if(rdfNamespace != null)
      trueArgs[1] = createResourceTerm (rdfNamespace);
    else
      trueArgs[1] = new VariableTerm ("_");
    org.semanticweb.lp.Literal[] head = {new org.semanticweb.lp.Literal ("true", trueArgs)};
    ProgramClause f = new ProgramClause (head,null);
    // writer.println("% RDF Statment: " + s + " in " + rdfNamespace);
    writer.println (f.toPrologString (new ArrayList (0)));
  }

}
