package org.semanticweb.triple.inference;

/*
 *  Name: SimpleEvaluatorConfig.java
 *  Version: 1.0
 *  Purpose:
 *  System-Wide Configuration Data
 */
import java.io.PrintStream;
import org.semanticweb.triple.inference.builtins.Add;
import org.semanticweb.triple.inference.builtins.atmsedge;
import org.semanticweb.triple.inference.builtins.between;
import org.semanticweb.triple.inference.builtins.Builtin;
import org.semanticweb.triple.inference.builtins.Concat;
import org.semanticweb.triple.inference.builtins.ends;
import org.semanticweb.triple.inference.builtins.Equal;
import org.semanticweb.triple.inference.builtins.Evaluable;
import org.semanticweb.triple.inference.builtins.greater;
import org.semanticweb.triple.inference.builtins.greaterorequal;
import org.semanticweb.triple.inference.builtins.isConst;
import org.semanticweb.triple.inference.builtins.isNum;
import org.semanticweb.triple.inference.builtins.isString;
import org.semanticweb.triple.inference.builtins.less;
import org.semanticweb.triple.inference.builtins.lessorequal;
import org.semanticweb.triple.inference.builtins.lives;
import org.semanticweb.triple.inference.builtins.rule;
import org.semanticweb.triple.inference.builtins.starts;
import org.semanticweb.triple.inference.builtins.string2double;
import org.semanticweb.triple.inference.builtins.subs;
import org.semanticweb.triple.inference.builtins.Write;

/**
 *  Description of the Class
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public class SimpleEvaluatorConfig {

    final static int WELLFOUNDED = 1;
    final static int ALTERNATINGFIXPOINT = 2;
    final static int DYNAMIC = 3;
    final static int NAIVE = 4;

    /**
     *  Description of the Field
     */
    public final static int STRINGSTART = 100000;

    /**
     *  Description of the Field
     */
    public DBInterface DBDEFAULT = new DB();
    // facts stored in RAM
    //public DBInterface DBDEFAULT = new SQL("org.gjt.mm.mysql.Driver", "jdbc:mysql://localhost/db"); // facts stored in mysql database

    //public DBInterface DBDEFAULT = new SQL("jdbc.idbDriver", "jdbc:idb=facts.prp"); // facts stored in idb database

    /**
     *  Description of the Field
     */
    public DBInterface DATABASE;

    //public Symbolhandler SYMBOLHANDLERDEFAULT = new SymbolhandlerAVL(); // symbols handled in RAM
    /**
     *  Description of the Field
     */
    public Symbolhandler SYMBOLHANDLERDEFAULT = new SymbolhandlerHash();
    // symbols handled in RAM

    //public Symbolhandler SYMBOLHANDLERDEFAULT = (Symbolhandler)DBDEFAULT; // symbols handled in database
    /**
     *  Description of the Field
     */
    public Symbolhandler SYMBOLHANDLER;

    /**
     *  Description of the Field
     */
    public PrintStream RULEOUTPUTDEFAULT = null;
    /**
     *  Description of the Field
     */
    public PrintStream RULEOUTPUT;

    /**
     *  Description of the Field
     */
    public int EVALUATIONMETHODDEFAULT = WELLFOUNDED;
    /**
     *  Description of the Field
     */
    public int EVALUATIONMETHOD;

    /**
     *  Description of the Field
     */
    public boolean[] OPTIMIZATIONDEFAULT = {true, false, false};
    /**
     *  Description of the Field
     */
    public boolean[] OPTIMIZATION;


    /**
     *  Constructor for the SimpleEvaluatorConfig object
     */
    public SimpleEvaluatorConfig() {
        setEvaluationmethodDefault();
        setSymbolhandlerDefault();
        setDatabaseDefault();
        setOptimizationDefault();
    }


    /**
     *  Gets the builtinList attribute of the SimpleEvaluatorConfig object
     *
     *@param  eval  Description of the Parameter
     *@return       The builtinList value
     */
    public Builtin[] getBuiltinList(SimpleEvaluator eval) {
        Builtin[] list = {
                new Builtin("Add", 3, new org.semanticweb.triple.inference.builtins.Add()),
                new Builtin("evaluable_", 2, new org.semanticweb.triple.inference.builtins.Evaluable()),
                new Builtin("Concat", 3, new org.semanticweb.triple.inference.builtins.Concat(eval)),
                new Builtin("unify", 2, new org.semanticweb.triple.inference.builtins.Equal()),
                new Builtin("isNum", 1, new org.semanticweb.triple.inference.builtins.isNum()),
                new Builtin("isString", 1, new org.semanticweb.triple.inference.builtins.isString()),
                new Builtin("isConst", 1, new org.semanticweb.triple.inference.builtins.isConst()),
                new Builtin("less", 2, new org.semanticweb.triple.inference.builtins.less()),
                new Builtin("lessorequal", 2, new org.semanticweb.triple.inference.builtins.lessorequal()),
                new Builtin("greater", 2, new org.semanticweb.triple.inference.builtins.greater()),
                new Builtin("greaterorequal", 2, new org.semanticweb.triple.inference.builtins.greaterorequal()),
                new Builtin("string2double", 2, new org.semanticweb.triple.inference.builtins.string2double(eval)),
        /*
         *  new Builtin("regexp",3, new edu.stanford.db.triple.inference.builtins.Regexp(eval)),
         */
                new Builtin("write", 1, new org.semanticweb.triple.inference.builtins.Write(eval)),
                new Builtin("between", 3, new org.semanticweb.triple.inference.builtins.between()),
        /*
         *  new Builtin("instantiated",1,new edu.stanford.db.triple.inference.builtins.instantiated())
         */
                new Builtin("starts", 2, new org.semanticweb.triple.inference.builtins.starts()),
                new Builtin("ends", 2, new org.semanticweb.triple.inference.builtins.ends()),
                new Builtin("lives", 2, new org.semanticweb.triple.inference.builtins.lives()),
                new Builtin("atms", 2, new org.semanticweb.triple.inference.builtins.atmsedge(eval)),
                new Builtin("subs", 3, new org.semanticweb.triple.inference.builtins.subs(eval)),
                new Builtin("rule", 3, new org.semanticweb.triple.inference.builtins.rule(eval))
                };
        return list;
    }


    /**
     *  Gets the database attribute of the SimpleEvaluatorConfig object
     *
     *@return    The database value
     */
    public DBInterface getDatabase() {
        return DATABASE;
    }


    /**
     *  Gets the evaluationmethod attribute of the SimpleEvaluatorConfig object
     *
     *@return    The evaluationmethod value
     */
    public int getEvaluationmethod() {
        return EVALUATIONMETHOD;
    }


    /**
     *  result[0]: propagating constants in rules result[1]: propagating
     *  constants in rules and facts. This may need too much memory. result[2]:
     *  poor mans rule optimization. AddFact, DeleteFact, AddFacts and
     *  dynamically added facts in a query does not work! This may need too much
     *  memory if configured without database.
     *
     *@return    The optimization value
     */
    public boolean[] getOptimization() {
        return OPTIMIZATION;
    }


    /**
     *  Gets the ruleoutput attribute of the SimpleEvaluatorConfig object
     *
     *@return    The ruleoutput value
     */
    public PrintStream getRuleoutput() {
        return RULEOUTPUT;
    }


    /**
     *  Gets the symbolhandler attribute of the SimpleEvaluatorConfig object
     *
     *@return    The symbolhandler value
     */
    public Symbolhandler getSymbolhandler() {
        return SYMBOLHANDLER;
    }


    /**
     *  Sets the database attribute of the SimpleEvaluatorConfig object
     *
     *@param  DATABASE  The new database value
     */
    public void setDatabase(DBInterface DATABASE) {
        this.DATABASE = DATABASE;
    }


    /**
     *  Sets the databaseDefault attribute of the SimpleEvaluatorConfig object
     */
    public void setDatabaseDefault() {
        DATABASE = DBDEFAULT;
    }


    /**
     *  Sets the evaluationmethod attribute of the SimpleEvaluatorConfig object
     *
     *@param  EVALUATIONMETHOD  The new evaluationmethod value
     */
    public void setEvaluationmethod(int EVALUATIONMETHOD) {
        this.EVALUATIONMETHOD = EVALUATIONMETHOD;
    }


    /**
     *  Sets the evaluationmethodDefault attribute of the SimpleEvaluatorConfig
     *  object
     */
    public void setEvaluationmethodDefault() {
        EVALUATIONMETHOD = EVALUATIONMETHODDEFAULT;
    }


    /**
     *  OPTIMIZATION[0]: propagating constants in rules OPTIMIZATION[1]:
     *  propagating constants in rules and facts. This may need too much memory.
     *  OPTIMIZATION[2]: poor mans rule optimization. AddFact, DeleteFact,
     *  AddFacts and dynamically added facts in a query does not work! This may
     *  need too much memory if configured without database.
     *
     *@param  OPTIMIZATION  The new optimization value
     */
    public void setOptimization(boolean[] OPTIMIZATION) {
        this.OPTIMIZATION = OPTIMIZATION;
    }


    /**
     *  Sets the optimizationDefault attribute of the SimpleEvaluatorConfig
     *  object
     */
    public void setOptimizationDefault() {
        OPTIMIZATION = OPTIMIZATIONDEFAULT;
    }


    /**
     *  Sets the ruleoutput attribute of the SimpleEvaluatorConfig object
     *
     *@param  RULEOUTPUT  The new ruleoutput value
     */
    public void setRuleoutput(PrintStream RULEOUTPUT) {
        this.RULEOUTPUT = RULEOUTPUT;
    }


    /**
     *  Sets the ruleoutputDefault attribute of the SimpleEvaluatorConfig object
     */
    public void setRuleoutputDefault() {
        RULEOUTPUT = RULEOUTPUTDEFAULT;
    }


    /**
     *  Sets the symbolhandler attribute of the SimpleEvaluatorConfig object
     *
     *@param  SYMBOLHANDLER  The new symbolhandler value
     */
    public void setSymbolhandler(Symbolhandler SYMBOLHANDLER) {
        this.SYMBOLHANDLER = SYMBOLHANDLER;
    }


    /**
     *  Sets the symbolhandlerDefault attribute of the SimpleEvaluatorConfig
     *  object
     */
    public void setSymbolhandlerDefault() {
        SYMBOLHANDLER = SYMBOLHANDLERDEFAULT;
    }
}
