package org.semanticweb.triple.inference;



import java.io.*;
import java.util.*;
import java.net.*;

/**
 *  Description of the Class
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public abstract class Term {
    /**
     *  Description of the Field
     */
    protected static int anz = 0;
    // number of terms created up to now
    /**
     *  Description of the Field
     */
    public boolean ground = true;
    // shows whether term contains variables or not
    /**
     *  Description of the Field
     */
    protected int groundlevel = -1;
    // gives the level until term is ground
    /**
     *  Description of the Field
     */
    public Term pars[] = null;
    // sub terms
    private Object attached = null;

    // attach any Java object


    /**
     *  Terms are the sub term
     *
     *@param  terms  is array of sub terms.
     */
    public Term(Term terms[]) {
        int i;
        ground = true;
        pars = new Term[terms.length];
        if (terms.length > 0) {
            groundlevel = terms[0].groundlevel + 1;
        }
        for (i = 0; i < terms.length; i++) {
            if (!terms[i].ground) {
                ground = false;
            }
            if (terms[i].groundlevel + 1 < groundlevel) {
                groundlevel = terms[i].groundlevel + 1;
            }
            pars[i] = terms[i];
        }
    }



    /**
     *  Constructs an empty term
     *
     *@param  arity   is the number of sub terms.
     *@param  ground  Description of the Parameter
     *@ground         says if the term can be treated as a ground term.
     */

    public Term(int arity, boolean ground) {
        anz++;
        // creating terms
        this.ground = ground;
        pars = new Term[arity];
    }


    /**
     *  Returns the number of sub terms
     *
     *@return    Description of the Return Value
     */
    public int anzpars() {
        if (pars == null) {
            return 0;
        }
        else {
            return pars.length;
        }
    }


    /**
     *  Attach any Java object to the term
     *
     *@param  o  Description of the Parameter
     */
    public void Attach(Object o) {
        attached = o;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public Term Clone() {
        // clone the terms and arguments
        int i;
        Term t;
        t = this.CloneFlat();
        for (i = 0; i < pars.length; i++) {
            t.pars[i] = pars[i].Clone();
        }
        return t;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public abstract Term CloneFlat();


    // clone the term, but not the arguments

    /**
     *  Compare two terms and parameters Delivers -1 if this < t 0 if this == t
     *  1 if this > t
     *
     *@param  t  if the Term t is compared with.
     *@return    the comparision result.
     */

    public int Compare(Term t) {
        int cmp;
        int i;
        if ((Type() != t.Type()) || (pars.length != t.pars.length)) {
            return CompareTypes(t);
        }
        else {
            cmp = CompareFlat(t);
            if (cmp == 0) {
                for (i = 0; i < pars.length; i++) {
                    cmp = pars[i].Compare(t.pars[i]);
                    if (cmp != 0) {
                        return cmp;
                    }
                }
            }
            return cmp;
        }
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    protected int CompareEqual(Term t) {
        return Compare(t);
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    protected abstract int CompareFlat(Term t);

    // compares two terms without sub terms


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    protected int CompareTypes(Term t) {
        // vergleicht zwei einzelne Terme (ohne Unterterme) aufgrund deren Typen und Stelligkeit
        // liefert die Werte -1 falls this < t, 0 falls this = t, 1 falls this > t
        int res;
        res = (Type() <= t.Type()) ? (Type() < t.Type() ? -1 : 0) : 1;
        return (res != 0) ? res : ((pars.length <= t.pars.length) ? ((pars.length < t.pars.length) ? -1 : 0) : 1);
    }


    /**
     *  Creates a term which unifies with this and t
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    public Term Generalize(Term t) {
        // erzeugt einen Term der sowohl mit this als auch mit t unifizierbar ist
        // Dabei muss this und t unifizierbar sein
        Term t1 = null;
        int i;
        boolean ground = true;
        if (t instanceof Variable) {
            t1 = t.CloneFlat();
        }
        else {
            t1 = this.CloneFlat();
        }
        for (i = 0; i < pars.length; i++) {
            t1.pars[i] = this.pars[i].Generalize(t.pars[i]);
            if (!t1.pars[i].ground) {
                ground = false;
            }
        }
        t1.ground = t1.ground && ground;
        return t1;
    }


    /**
     *  give the attached Java object
     *
     *@return    The attached value
     */
    public Object getAttached() {
        return attached;
    }


    /**
     *  Creates a flat format out of the term
     *
     *@return    Description of the Return Value
     */
    public String internalize() {
        int i;
        String s;
        s = InternalizeFlat();
        if (pars.length > 0) {
            s = s + "l" + String.valueOf(pars.length);
            for (i = 0; i < pars.length; i++) {
                s = s + pars[i].internalize();
            }
        }
        return s;
    }


    /**
     *  Writes internal format to printstream
     *
     *@param  p  Description of the Parameter
     */
    public void internalize(PrintStream p) {
        p.print(internalize());
    }


    // Ausgabe des Terms als String, p,f,s sind Symboltabellen fr Prdikate,Funktionssymbole,Strings
    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    protected abstract String InternalizeFlat();


    /**
     *  Is it a constant?
     *
     *@return    The constTerm value
     */
    public boolean isConstTerm() {
        return false;
    }


    /**
     *  Is the term ground ?
     *
     *@param  v  Description of the Parameter
     *@return    The ground value
     */

    public boolean isGround(long v) {
        // v hat an der Stelle Bit i gesetzt, wenn Variable i nicht substituiert sein soll
        // isGround gibt an, ob der Term unter zustzlicher Substitution der
        // Variablen in v ground ist
        int i;
        if (ground) {
            return true;
        }
        else if (this instanceof Variable) {
            if (((1L << (((Variable) this).symbol)) & v) == 0) {
                return true;
            }
            else {
                return false;
            }
        }
        else {
            for (i = 0; i < pars.length; i++) {
                if (!pars[i].isGround(v)) {
                    return false;
                }
            }
        }
        return true;
    }


    /**
     *  Is it a number?
     *
     *@return    The numTerm value
     */
    public boolean isNumTerm() {
        return false;
    }


    /**
     *  Is it a string?
     *
     *@return    The stringTerm value
     */
    public boolean isStringTerm() {
        return false;
    }


    /**
     *  Is it a variable?
     *
     *@return    The variable value
     */
    public boolean isVariable() {
        return false;
    }


    /**
     *  Matches this with t. t must be a ground term. The substituting terms are
     *  attached to the variables
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    public boolean Match(Term t) {
        // Matcht den Term this mit dem Grundterm t. Die Substitutionen fr die Variablen werden an das
        // Feld subsby der Variablen in this angehngt
        int i;
        boolean res = true;
        //if (t.ground) {
        if ((CompareTypes(t) != 0) || (CompareFlat(t) != 0)) {
            return false;
        }
        else {
            for (i = 0; i < pars.length; i++) {
                res = pars[i].Match(t.pars[i]);
                if (!res) {
                    return res;
                }
            }
            return res;
        }
    }


    /**
     *  Description of the Method
     *
     *@param  v  Description of the Parameter
     *@return    Description of the Return Value
     */
    protected boolean occurs(Variable v) {
        // Occurence Check fr Variablen: true, falls Variable v in this auftritt. Dabei sind
        // die Variablensymbole magebend
        int i;
        boolean res = false;
        if (!ground) {
            for (i = 0; i < pars.length; i++) {
                res = pars[i].occurs(v);
                if (res) {
                    return res;
                }
            }
        }
        return res;
    }


    /**
     *  Prints term to printstream
     *
     *@param  p  Description of the Parameter
     */
    public void print(PrintStream p) {
        p.print(toString());
    }


    /**
     *  Description of the Method
     *
     *@param  pr  Description of the Parameter
     *@param  p   Description of the Parameter
     *@param  f   Description of the Parameter
     *@param  st  Description of the Parameter
     */
    public void print(PrintStream pr, String p[], String f[], String st[]) {
        pr.print(toString(p, f, st));
    }


    /**
     *  Substitutes a variable in the term by the term attached to the variable
     *  as substitution
     *
     *@return    Description of the Return Value
     */
    public Term Substitute() {
        // Durchfhrung einer Substitution, d.h. die Variable wird durch ihren
        // Substituenden im Term ersetzt
        // Dabei werden alle oberen Terme (oberhalb von Variablen) neu generiert
        Term t;
        int i;
        if (ground) {
            return this;
        }
        else {
            t = this.CloneFlat();
            t.ground = true;
            for (i = 0; i < pars.length; i++) {
                t.pars[i] = pars[i].Substitute();
                if (!t.pars[i].ground) {
                    t.ground = false;
                }
            }
            return t;
        }
    }


    /**
     *  Creates another string representation
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        // Ausgabe eines Terms mit Untertermen als String
        int i;
        String s;
        s = toStringFlat();
        if (pars.length > 0) {
            s = s.concat("(");
            for (i = 0; i < pars.length; i++) {
                s = s.concat(pars[i].toString());
                if (i < pars.length - 1) {
                    s = s.concat(",");
                }
            }
            s = s.concat(")");
        }
        return s;
    }


    /**
     *  Creates a string representation. p,f,st contain names for predicate,
     *  function, string symbols
     *
     *@param  p   Description of the Parameter
     *@param  f   Description of the Parameter
     *@param  st  Description of the Parameter
     *@return     Description of the Return Value
     */
    public String toString(String p[], String f[], String st[]) {
        // Ausgabe eines Terms mit Untertermen als String, p,f,s sind Symboltabellen fr
        // Prdikatsymbole, Funktionssymbole, Strings
        int i;
        String s;
        // System.out.println("Term.toString");
        s = toStringFlat(p, f, st);
        if (pars.length > 0) {
            s = s.concat("(");
            for (i = 0; i < pars.length; i++) {
                s = s.concat(pars[i].toString(p, f, st));
                if (i < pars.length - 1) {
                    s = s.concat(",");
                }
            }
            s = s.concat(")");
        }
        return s;
    }


    // -1 if this < t, 0 if this = t, 1 if this > t

    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    protected abstract String toStringFlat();


    // Ausgabe eines Terms als String

    /**
     *  Description of the Method
     *
     *@param  p  Description of the Parameter
     *@param  f  Description of the Parameter
     *@param  s  Description of the Parameter
     *@return    Description of the Return Value
     */
    protected abstract String toStringFlat(String p[], String f[], String s[]);


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    protected int type() {
        return Type();
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public abstract int Type();

    // each different term type is associated to a number


    /**
     *  Unifies this with t. The substituting terms are attached to the
     *  variables
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    public boolean Unify(Term t) {
        // Unifiziert zwei Terme. Die Substitutionen fr die Variablen werden an das
        // Feld subsby der Variablen angehngt
        int i;
        boolean res = true;
        if (t instanceof Variable) {
            return t.Unify(this);
        }
        else
                if ((CompareTypes(t) != 0) || (CompareFlat(t) != 0)) {
            return false;
        }
        else {
            for (i = 0; i < pars.length; i++) {
                res = pars[i].Unify(t.pars[i]);
                if (!res) {
                    return res;
                }
            }
            return res;
        }
    }
}
