package org.semanticweb.triple.inference.builtins;

import java.io.*;
import java.lang.Math;
import org.semanticweb.triple.inference.ATMS;
import org.semanticweb.triple.inference.Atom;
import org.semanticweb.triple.inference.Variable;
import org.semanticweb.triple.inference.Term;
import org.semanticweb.triple.inference.StringTerm;
import org.semanticweb.triple.inference.BuiltinFunc;
import org.semanticweb.triple.inference.Filter;
import org.semanticweb.triple.inference.GroundAtom;
import org.semanticweb.triple.inference.NumTerm;
import org.semanticweb.triple.inference.Rule;
import org.semanticweb.triple.inference.RuleSet;
import org.semanticweb.triple.inference.SimpleEvaluator;
import org.semanticweb.triple.inference.SimpleEvaluatorConfig;
import org.semanticweb.triple.inference.ConstTerm;

/**
 *  Copyright  Juergen Angele and Stefan Decker, Institut AIFB, University of
 *  Karlsruhe. All Rights Reserved.
 *
 *@author     Administrator
 *@created    January 17, 2002
 */

public class atmsedge extends BuiltinFunc {

    /**
     *  Constructor for the atmsedge object
     */
    public atmsedge() {
        super();
    }


    /**
     *  Constructor for the atmsedge object
     *
     *@param  eval  Description of the Parameter
     */
    public atmsedge(SimpleEvaluator eval) {
        super(eval);
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     */
    public void eval(Atom t) {
//System.out.println("-> atmsedge");
        GroundAtom g;
        ATMS a;
        Rule r;
        Filter f;
        int anz = 0;
        int i;
        RuleSet rs = eval.getRuleset();

        for (r = rs.rules; r != null; r = r.next1) {
            for (g = r.hrelation.First(); g != null; g = r.hrelation.Next()) {
                g.mark = r.no;
            }
        }
        // suche alle Substitutionen aller Regeln durch
        for (r = rs.rules; r != null; r = r.next1) {
            if (r.facts) {
                for (g = r.heads[0].up.First(); g != null; g = r.heads[0].up.Next()) {
                    if (isstart(t, g)) {
                        g.mark = r.no;
                        for (a = g.supported; a != null; a = a.next) {
                            follow(g, a.atom, t);
                        }
                    }
                }

            }
            else {
                for (g = r.hrelation.First(); g != null; g = r.hrelation.Next()) {
                    if (isstart(t, g)) {
                        for (a = g.supported; a != null; a = a.next) {
                            follow(g, a.atom, t);
                        }
                    }
                }
            }
        }
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    public boolean evaluable(Atom t) {
        if (!((t.terms[0] instanceof Variable) || (t.terms[0] instanceof NumTerm))) {
            return false;
        }
        if (!((t.terms[1] instanceof Variable) || (t.terms[1] instanceof NumTerm))) {
            return false;
        }
        return true;
    }


    /**
     *  Description of the Method
     *
     *@param  a  Description of the Parameter
     *@param  b  Description of the Parameter
     *@param  t  Description of the Parameter
     */
    private void follow(GroundAtom a, GroundAtom b, Atom t) {
        ATMS at;
        Atom g;
        if (b.mark == 0) {
            for (at = b.supported; at != null; at = at.next) {
                follow(a, at.atom, t);
            }
        }
        else if (isend(t, b)) {
            g = new Atom(2);
            g.terms[0] = new NumTerm(a.num);
            g.terms[1] = new NumTerm(b.num);
            insert(g);
            g = null;
        }
    }


    /**
     *  Gets the string attribute of the atmsedge object
     *
     *@param  t  Description of the Parameter
     *@return    The string value
     */
    private String getString(org.semanticweb.triple.inference.Term t) {
        if (t instanceof org.semanticweb.triple.inference.StringTerm) {
            return ((org.semanticweb.triple.inference.StringTerm) t).s;
        }
        else {
            return eval.getString(((org.semanticweb.triple.inference.ConstTerm) t).symbol);
        }

    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@param  g  Description of the Parameter
     *@return    Description of the Return Value
     */
    private boolean isend(Atom t, GroundAtom g) {

        if ((t.terms[1] instanceof NumTerm) && (((NumTerm) t.terms[1]).zahl != g.num)) {
            return false;
        }
        return true;
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@param  g  Description of the Parameter
     *@return    Description of the Return Value
     */
    private boolean isstart(Atom t, GroundAtom g) {
        if ((t.terms[0] instanceof NumTerm) && (((NumTerm) t.terms[0]).zahl != g.num)) {
            return false;
        }
        return true;
    }


    /**
     *  Gets the string attribute of the atmsedge object
     *
     *@param  t  Description of the Parameter
     *@return    The string value
     */
    private boolean isString(org.semanticweb.triple.inference.Term t) {
        if ((t instanceof StringTerm) ||
                ((t instanceof org.semanticweb.triple.inference.ConstTerm && ((org.semanticweb.triple.inference.ConstTerm) t).symbol >= SimpleEvaluatorConfig.STRINGSTART))) {
            return true;
        }
        return false;
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    private String subs(GroundAtom t) {
        String s = "";
        for (int i = 0; i < t.terms.length; i++) {
            org.semanticweb.triple.inference.Term arg = t.terms[i];
            org.semanticweb.lp.Term f = eval.toSimpleTerm(arg, eval.getSymbolhandler());
            s += f.toString().replace('\"', ' ');
            if (i < t.terms.length - 1) {
                s += ",";
            }
        }
        return s;
    }
}
