package org.semanticweb.triple.inference.builtins;

/*
 *  Name: subs.java
 *  Version: 1.0
 *  Purpose: outputs the variable substitutions of a rule
 *  History:
 */
import java.io.*;
import java.lang.Math;
import org.semanticweb.triple.inference.Atom;
import org.semanticweb.triple.inference.Variable;
import org.semanticweb.triple.inference.Term;
import org.semanticweb.triple.inference.StringTerm;
import org.semanticweb.triple.inference.BuiltinFunc;
import org.semanticweb.triple.inference.GroundAtom;
import org.semanticweb.triple.inference.NumTerm;
import org.semanticweb.triple.inference.Rule;
import org.semanticweb.triple.inference.RuleSet;
import org.semanticweb.triple.inference.SimpleEvaluator;
import org.semanticweb.triple.inference.SimpleEvaluatorConfig;
import org.semanticweb.triple.inference.ConstTerm;

/**
 *  Description of the Class
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public class subs extends BuiltinFunc {

    /**
     *  Constructor for the subs object
     */
    public subs() {
        super();
    }


    /**
     *  Constructor for the subs object
     *
     *@param  eval  Description of the Parameter
     */
    public subs(SimpleEvaluator eval) {
        super(eval);
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@param  g  Description of the Parameter
     *@return    Description of the Return Value
     */
    private boolean check(Atom t, GroundAtom g) {
        if (isString(t.terms[1]) && !(getString(t.terms[1])).equals(subs(g))) {
            return false;
        }
        if ((t.terms[2] instanceof NumTerm) && (((NumTerm) t.terms[2]).zahl != g.num)) {
            return false;
        }
        return true;
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     */
    public void eval(Atom t) {
        GroundAtom g;
        Atom h;
        Rule r;
        int z = -1;

        RuleSet rs = eval.getRuleset();
        if (t.terms[0] instanceof NumTerm) {
            z = (int) ((NumTerm) t.terms[0]).zahl;
        }

        if (z != -1) {
            for (r = rs.rules; r != null; r = r.next1) {
                if (z == r.no) {
                    if (r.facts) {
                        for (g = r.heads[0].up.First(); g != null; g = r.heads[0].up.Next()) {
                            if (check(t, g)) {
                                h = new Atom(3);
                                h.terms[0] = new NumTerm(r.no);
                                h.terms[1] = new ConstTerm(eval.setString(subs(g)));
                                h.terms[2] = new NumTerm(g.num);
                                insert(h);
                            }
                        }

                    }
                    else {
                        for (g = r.hrelation.First(); g != null; g = r.hrelation.Next()) {
                            if (check(t, g)) {
                                h = new Atom(3);
                                h.terms[0] = new NumTerm(r.no);
                                h.terms[1] = new ConstTerm(eval.setString(subs(g)));
                                h.terms[2] = new NumTerm(g.num);
                                insert(h);
                            }
                        }
                    }
                }
            }
        }
        else {
            for (r = rs.rules; r != null; r = r.next1) {
                if (r.facts) {
                    for (g = r.heads[0].up.First(); g != null; g = r.heads[0].up.Next()) {
                        if (check(t, g)) {
                            h = new Atom(3);
                            h.terms[0] = new NumTerm(r.no);
                            h.terms[1] = new ConstTerm(eval.setString(subs(g)));
                            h.terms[2] = new NumTerm(g.num);
                            insert(h);
                        }
                    }

                }
                else {
                    for (g = r.hrelation.First(); g != null; g = r.hrelation.Next()) {
                        if (check(t, g)) {
                            h = new Atom(3);
                            h.terms[0] = new NumTerm(r.no);
                            h.terms[1] = new ConstTerm(eval.setString(subs(g)));
                            h.terms[2] = new NumTerm(g.num);
                            insert(h);
                        }
                    }
                }
            }
        }
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    public boolean evaluable(Atom t) {
        if (!((t.terms[0] instanceof Variable) || (t.terms[0] instanceof NumTerm))) {
            return false;
        }
        if (!((t.terms[1] instanceof Variable) || (isString(t.terms[1])))) {
            return false;
        }
        if (!((t.terms[2] instanceof Variable) || (t.terms[2] instanceof NumTerm))) {
            return false;
        }
        return true;
    }


    /**
     *  Gets the string attribute of the subs object
     *
     *@param  t  Description of the Parameter
     *@return    The string value
     */
    private String getString(org.semanticweb.triple.inference.Term t) {
        if (t instanceof org.semanticweb.triple.inference.StringTerm) {
            return ((org.semanticweb.triple.inference.StringTerm) t).s;
        }
        else {
            return eval.getString(((org.semanticweb.triple.inference.ConstTerm) t).symbol);
        }

    }


    /**
     *  Gets the string attribute of the subs object
     *
     *@param  t  Description of the Parameter
     *@return    The string value
     */
    private boolean isString(org.semanticweb.triple.inference.Term t) {
        if ((t instanceof StringTerm) ||
                ((t instanceof org.semanticweb.triple.inference.ConstTerm && ((org.semanticweb.triple.inference.ConstTerm) t).symbol >= SimpleEvaluatorConfig.STRINGSTART))) {
            return true;
        }
        return false;
    }


    /**
     *  Description of the Method
     *
     *@param  t  Description of the Parameter
     *@return    Description of the Return Value
     */
    private String subs(GroundAtom t) {
        String s = "";
        for (int i = 0; i < t.terms.length; i++) {
            org.semanticweb.triple.inference.Term arg = t.terms[i];
            org.semanticweb.lp.Term f = eval.toSimpleTerm(arg, eval.getSymbolhandler());
            s += f.toString().replace('\"', ' ');
            if (i < t.terms.length - 1) {
                s += ",";
            }
        }
        return s;
    }
}
