/*
 * Context.java
 *
 * Created on January 25, 2002, 4:14 PM
 */

package org.semanticweb.triple.parser;

import java.util.*;


/**
 *
 * @author  sintek
 * @version 
 */
public class Context {

  Callback _callback;
  HashMap _abbreviations; // String -> LTerm
  LTerm _model;
  
  /** Creates new Context */
  public Context(Callback callback) {
    _callback = callback;
    _abbreviations = new HashMap();
    _model = new LVariable(); // anonymous variable!
  }
  
  public HashMap getAbbreviations() {
    return _abbreviations;
  }
  
  public LTerm getModel() {
    return _model;
  }
  
  public Context clone(ArrayList variables, LTerm model) {
    Context newContext = new Context(_callback);
    HashMap newAbbreviations = (HashMap)_abbreviations.clone();
    addVariables(variables, newAbbreviations);
    // System.out.println("new abbreviations: " + newAbbreviations);
    newContext._abbreviations = newAbbreviations;
    newContext._model = model.normalize(newAbbreviations);
    return newContext;
  }
  
  public static void addVariables(ArrayList variables, HashMap map) {
    if (variables != null) {
      for (Iterator varIter = variables.iterator(); varIter.hasNext();) {
        LTerm var = (LTerm)varIter.next();
        String varname = null;
        LTerm varterm = null;
        if (var instanceof LSymbol) {
          varname = ((LSymbol)var).getSymbol();
          varterm = new LVariable(varname);
        } else if (var instanceof LVariable) {
          varname = ((LVariable)var).getVariable();
          varterm = (LVariable)var;
        } // else warning/error ... !!!
        map.put(varname, varterm);
      }
    }
  }
  
  public void nextClause(LTerm term) {
    // if term is a declaration, update context,
    // else do callback
    if (term instanceof LStruct) {
      LStruct declaration = (LStruct)term;
      LTerm functor = declaration.getFunctor();
      if (functor instanceof LSymbol) {
        String functorString = ((LSymbol)functor).getSymbol();
        if (functorString.equals(":=")) {
          ArrayList args = declaration.getArgs();
          if (args.size() == 2) {
            // update abbreviations
            LTerm abbrev = (LTerm)args.get(0);
            LTerm value = (LTerm)args.get(1);
            if (abbrev instanceof LSymbol) {
              if (occurs((LSymbol)abbrev, value))
                System.err.println("WARNING: abbreviation occurs in value: " +
                  abbrev + " := " + value);
              _abbreviations.put(((LSymbol)abbrev).getSymbol(), 
                value.normalize(_abbreviations));
            } else
              System.err.println("WARNING: malformed abbreviation ignored: " + term);
            return;
          }
        }
      }
    }
    // do callback
    _callback.nextClause(term.normalize(_abbreviations), this);
  }
  
  public boolean occurs(LSymbol symbol, LTerm term) {
    // checks if symbol occurs in term
    // System.out.println("occurs : " + symbol + " in " + term);
    if (term instanceof LSymbol)
      return symbol.equals(term);
    else if (term instanceof LStruct) {
      LStruct struct = (LStruct)term;
      if (occurs(symbol, struct.getFunctor()))
        return true;
      else {
        for (Iterator argsIter = struct.getArgs().iterator(); argsIter.hasNext();) {
          LTerm arg = (LTerm)argsIter.next();
          if (occurs(symbol, arg))
            return true;
        }
        return false;
      }
    } else
      return false;
  }

}
